
/* IVars.c : Instruction tables -- pointers to functions.
 *
 * Copyright 1996 Rui Fernando Ferreira Ribeiro.
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */

/* aprox. 1500 opcodes.
		    20 duplicates (10+10 w/ IX e IY extensions)  ]
*/

/* Funcoes de execucao */
static void execute_IX();
static void execute_IY();
static void execute_CB();
static void execute_IXCB();
static void execute_IYCB();
static void execute_ED();

/* arrays of pointers to functions */
static void (*instruc_tabl[256])() =
{
/* 00 */   nop         ,
/* 01 */   ld_bc_nn    ,
/* 02 */   ld_pbc_a    ,
/* 03 */   inc_bc      ,
/* 04 */   inc_b       ,
/* 05 */   dec_b       ,
/* 06 */   ld_b_n      ,
/* 07 */   rlca        ,
/* 08 */   ex_af_af2   ,
/* 09 */   add_hl_bc   ,
/* 0A */   ld_a_pbc    ,
/* 0B */   dec_bc      ,
/* 0C */   inc_c       ,
/* 0D */   dec_c       ,
/* 0E */   ld_c_n      ,
/* 0F */   rrca        ,
/* 10 */   djnz_e      ,
/* 11 */   ld_de_nn    ,
/* 12 */   ld_pde_a    ,
/* 13 */   inc_de      ,
/* 14 */   inc_d       ,
/* 15 */   dec_d       ,
/* 16 */   ld_d_n      ,
/* 17 */   rla         ,
/* 18 */   jr_e        ,
/* 19 */   add_hl_de   ,
/* 1A */   ld_a_pde    ,
/* 1B */   dec_de      ,
/* 1C */   inc_e       ,
/* 1D */   dec_e       ,
/* 1E */   ld_e_n      ,
/* 1F */   rra         ,
/* 20 */   jr_nz_e     ,
/* 21 */   ld_hl_nn    ,
/* 22 */   ld_pnn_hl   ,
/* 23 */   inc_hl      ,
/* 24 */   inc_h       ,
/* 25 */   dec_h       ,
/* 26 */   ld_h_n      ,
/* 27 */   daa         ,
/* 28 */   jr_z_e      ,
/* 29 */   add_hl_hl   ,
/* 2A */   ld_hl_pnn   ,
/* 2B */   dec_hl      ,
/* 2C */   inc_l       ,
/* 2D */   dec_l       ,
/* 2E */   ld_l_n      ,
/* 2F */   cpl         ,
/* 30 */   jr_nc_e     ,
/* 31 */   ld_sp_nn    ,
/* 32 */   ld_pnn_a    ,
/* 33 */   inc_sp      ,
/* 34 */   inc_phl     ,
/* 35 */   dec_phl     ,
/* 36 */   ld_phl_n    ,
/* 37 */   scf         ,
/* 38 */   jr_c_e      ,
/* 39 */   add_hl_sp   ,
/* 3A */   ld_a_pnn    ,
/* 3B */   dec_sp      ,
/* 3C */   inc_a       ,
/* 3D */   dec_a       ,
/* 3E */   ld_a_n      ,
/* 3F */   ccf         ,
/* 40 */   ld_b_b      ,
/* 41 */   ld_b_c      ,
/* 42 */   ld_b_d      ,
/* 43 */   ld_b_e      ,
/* 44 */   ld_b_h      ,
/* 45 */   ld_b_l      ,
/* 46 */   ld_b_phl    ,
/* 47 */   ld_b_a      ,
/* 48 */   ld_c_b      ,
/* 49 */   ld_c_c      ,
/* 4A */   ld_c_d      ,
/* 4B */   ld_c_e      ,
/* 4C */   ld_c_h      ,
/* 4D */   ld_c_l      ,
/* 4E */   ld_c_phl    ,
/* 4F */   ld_c_a      ,
/* 50 */   ld_d_b      ,
/* 51 */   ld_d_c      ,
/* 52 */   ld_d_d      ,
/* 53 */   ld_d_e      ,
/* 54 */   ld_d_h      ,
/* 55 */   ld_d_l      ,
/* 56 */   ld_d_phl    ,
/* 57 */   ld_d_a      ,
/* 58 */   ld_e_b      ,
/* 59 */   ld_e_c      ,
/* 5A */   ld_e_d      ,
/* 5B */   ld_e_e      ,
/* 5C */   ld_e_h      ,
/* 5D */   ld_e_l      ,
/* 5E */   ld_e_phl    ,
/* 5F */   ld_e_a      ,
/* 60 */   ld_h_b      ,
/* 61 */   ld_h_c      ,
/* 62 */   ld_h_d      ,
/* 63 */   ld_h_e      ,
/* 64 */   ld_h_h      ,
/* 65 */   ld_h_l      ,
/* 66 */   ld_h_phl    ,
/* 67 */   ld_h_a      ,
/* 68 */   ld_l_b      ,
/* 69 */   ld_l_c      ,
/* 6A */   ld_l_d      ,
/* 6B */   ld_l_e      ,
/* 6C */   ld_l_h      ,
/* 6D */   ld_l_l      ,
/* 6E */   ld_l_phl    ,
/* 6F */   ld_l_a      ,
/* 70 */   ld_phl_b    ,
/* 71 */   ld_phl_c    ,
/* 72 */   ld_phl_d    ,
/* 73 */   ld_phl_e    ,
/* 74 */   ld_phl_h    ,
/* 75 */   ld_phl_l    ,
/* 76 */   halt        ,
/* 77 */   ld_phl_a    ,
/* 78 */   ld_a_b      ,
/* 79 */   ld_a_c      ,
/* 7A */   ld_a_d      ,
/* 7B */   ld_a_e      ,
/* 7C */   ld_a_h      ,
/* 7D */   ld_a_l      ,
/* 7E */   ld_a_phl    ,
/* 7F */   ld_a_a      ,
/* 80 */   add_a_b     ,
/* 81 */   add_a_c     ,
/* 82 */   add_a_d     ,
/* 83 */   add_a_e     ,
/* 84 */   add_a_h     ,
/* 85 */   add_a_l     ,
/* 86 */   add_a_phl   ,
/* 87 */   add_a_a     ,
/* 88 */   adc_a_b     ,
/* 89 */   adc_a_c     ,
/* 8A */   adc_a_d     ,
/* 8B */   adc_a_e     ,
/* 8C */   adc_a_h     ,
/* 8D */   adc_a_l     ,
/* 8E */   adc_a_phl   ,
/* 8F */   adc_a_a     ,
/* 90 */   sub_b       ,
/* 91 */   sub_c       ,
/* 92 */   sub_d       ,
/* 93 */   sub_e       ,
/* 94 */   sub_h       ,
/* 95 */   sub_l       ,
/* 96 */   sub_phl     ,
/* 97 */   sub_a       ,
/* 98 */   sbc_a_b     ,
/* 99 */   sbc_a_c     ,
/* 9A */   sbc_a_d     ,
/* 9B */   sbc_a_e     ,
/* 9C */   sbc_a_h     ,
/* 9D */   sbc_a_l     ,
/* 9E */   sbc_a_phl   ,
/* 9F */   sbc_a_a     ,
/* A0 */   and_b       ,
/* A1 */   and_c       ,
/* A2 */   and_d       ,
/* A3 */   and_e       ,
/* A4 */   and_h       ,
/* A5 */   and_l       ,
/* A6 */   and_phl     ,
/* A7 */   and_a       ,
/* A8 */   xor_b       ,
/* A9 */   xor_c       ,
/* AA */   xor_d       ,
/* AB */   xor_e       ,
/* AC */   xor_h       ,
/* AD */   xor_l       ,
/* AE */   xor_phl     ,
/* AF */   xor_a       ,
/* B0 */   or_b        ,
/* B1 */   or_c        ,
/* B2 */   or_d        ,
/* B3 */   or_e        ,
/* B4 */   or_h        ,
/* B5 */   or_l        ,
/* B6 */   or_phl      ,
/* B7 */   or_a        ,
/* B8 */   cp_b        ,
/* B9 */   cp_c        ,
/* BA */   cp_d        ,
/* BB */   cp_e        ,
/* BC */   cp_h        ,
/* BD */   cp_l        ,
/* BE */   cp_phl      ,
/* BF */   cp_a        ,
/* C0 */   ret_nz      ,
/* C1 */   pop_bc      ,
/* C2 */   jp_nz_nn    ,
/* C3 */   jp_nn       ,
/* C4 */   call_nz_nn  ,
/* C5 */   push_bc     ,
/* C6 */   add_a_n     ,
/* C7 */   rst_0       ,
/* C8 */   ret_z       ,
/* C9 */   ret         ,
/* CA */   jp_z_nn     ,
/* CB */   execute_CB  ,
/* CC */   call_z_nn   ,
/* CD */   call_nn     ,
/* CE */   adc_a_n     ,
/* CF */   rst_8       ,
/* D0 */   ret_nc      ,
/* D1 */   pop_de      ,
/* D2 */   jp_nc_nn    ,
/* D3 */   out_pn_a    ,
/* D4 */   call_nc_nn  ,
/* D5 */   push_de     ,
/* D6 */   sub_n       ,
/* D7 */   rst_10      ,
/* D8 */   ret_c       ,
/* D9 */   exx         ,
/* DA */   jp_c_nn     ,
/* DB */   in_a_pn     ,
/* DC */   call_c_nn   ,
/* DD */   execute_IX  ,
/* DE */   sbc_a_n     ,
/* DF */   rst_18      ,
/* E0 */   ret_po      ,
/* E1 */   pop_hl      ,
/* E2 */   jp_po_nn    ,
/* E3 */   ex_psp_hl   ,
/* E4 */   call_po_nn  ,
/* E5 */   push_hl     ,
/* E6 */   and_n       ,
/* E7 */   rst_20      ,
/* E8 */   ret_pe      ,
/* E9 */   jp_hl       ,
/* EA */   jp_pe_nn    ,
/* EB */   ex_de_hl    ,
/* EC */   call_pe_nn  ,
/* ED */   execute_ED  ,
/* EE */   xor_n       ,
/* EF */   rst_28      ,
/* F0 */   ret_p       ,
/* F1 */   pop_af      ,
/* F2 */   jp_p_nn     ,
/* F3 */   di          ,
/* F4 */   call_p_nn   ,
/* F5 */   push_af     ,
/* F6 */   or_n        ,
/* F7 */   rst_30      ,
/* F8 */   ret_m       ,
/* F9 */   ld_sp_hl    ,
/* FA */   jp_m_nn     ,
/* FB */   ei          ,
/* FC */   call_m_nn   ,
/* FD */   execute_IY  ,
/* FE */   cp_n        ,
/* FF */   rst_38
};

static void (*instruc_tablIX[256])() =
{
/* 00 */   dummyIX     ,
/* 01 */   dummyIX     ,
/* 02 */   dummyIX     ,
/* 03 */   dummyIX     ,
/* 04 */   dummyIX     ,
/* 05 */   dummyIX     ,
/* 06 */   dummyIX     ,
/* 07 */   dummyIX     ,
/* 08 */   dummyIX     ,
/* 09 */   add_ix_bc   ,
/* 0A */   dummyIX     ,
/* 0B */   dummyIX     ,
/* 0C */   dummyIX     ,
/* 0D */   dummyIX     ,
/* 0E */   dummyIX     ,
/* 0F */   dummyIX     ,
/* 10 */   dummyIX     ,
/* 11 */   dummyIX     ,
/* 12 */   dummyIX     ,
/* 13 */   dummyIX     ,
/* 14 */   dummyIX     ,
/* 15 */   dummyIX     ,
/* 16 */   dummyIX     ,
/* 17 */   dummyIX     ,
/* 18 */   dummyIX     ,
/* 19 */   add_ix_de   ,
/* 1A */   dummyIX     ,
/* 1B */   dummyIX     ,
/* 1C */   dummyIX     ,
/* 1D */   dummyIX     ,
/* 1E */   dummyIX     ,
/* 1F */   dummyIX     ,
/* 20 */   dummyIX     ,
/* 21 */   ld_ix_nn    ,
/* 22 */   ld_pnn_ix   ,
/* 23 */   inc_ix      ,
/* 24 */   inc_Ix      ,
/* 25 */   dec_Ix      ,
/* 26 */   ld_Ix_n     ,
/* 27 */   dummyIX     ,
/* 28 */   dummyIX     ,
/* 29 */   add_ix_ix   ,
/* 2A */   ld_ix_pnn   ,
/* 2B */   dec_ix      ,
/* 2C */   inc_iX      ,
/* 2D */   dec_iX      ,
/* 2E */   ld_iX_n     ,
/* 2F */   dummyIX     ,
/* 30 */   dummyIX     ,
/* 31 */   dummyIX     ,
/* 32 */   dummyIX     ,
/* 33 */   dummyIX     ,
/* 34 */   inc_pix     ,
/* 35 */   dec_pix     ,
/* 36 */   ld_pix_n    ,
/* 37 */   dummyIX     ,
/* 38 */   dummyIX     ,
/* 39 */   add_ix_sp   ,
/* 3A */   dummyIX     ,
/* 3B */   dummyIX     ,
/* 3C */   dummyIX     ,
/* 3D */   dummyIX     ,
/* 3E */   dummyIX     ,
/* 3F */   dummyIX     ,
/* 40 */   dummyIX     ,
/* 41 */   dummyIX     ,
/* 42 */   dummyIX     ,
/* 43 */   dummyIX     ,
/* 44 */   ld_b_Ix     ,
/* 45 */   ld_b_iX     ,
/* 46 */   ld_b_pix    ,
/* 47 */   dummyIX     ,
/* 48 */   dummyIX     ,
/* 49 */   dummyIX     ,
/* 4A */   dummyIX     ,
/* 4B */   dummyIX     ,
/* 4C */   ld_c_Ix     ,
/* 4D */   ld_c_iX     ,
/* 4E */   ld_c_pix    ,
/* 4F */   dummyIX     ,
/* 50 */   dummyIX     ,
/* 51 */   dummyIX     ,
/* 52 */   dummyIX     ,
/* 53 */   dummyIX     ,
/* 54 */   ld_d_Ix     ,
/* 55 */   ld_d_iX     ,
/* 56 */   ld_d_pix    ,
/* 57 */   dummyIX     ,
/* 58 */   dummyIX     ,
/* 59 */   dummyIX     ,
/* 5A */   dummyIX     ,
/* 5B */   dummyIX     ,
/* 5C */   ld_e_Ix     ,
/* 5D */   ld_e_iX     ,
/* 5E */   ld_e_pix    ,
/* 5F */   dummyIX     ,
/* 60 */   ld_Ix_b     ,
/* 61 */   ld_Ix_c     ,
/* 62 */   ld_Ix_d     ,
/* 63 */   ld_Ix_e     ,
/* 64 */   ld_Ix_Ix    ,
/* 65 */   ld_Ix_iX    ,
/* 66 */   ld_h_pix    ,
/* 67 */   ld_Ix_a     ,
/* 68 */   ld_iX_b     ,
/* 69 */   ld_iX_c     ,
/* 6A */   ld_iX_d     ,
/* 6B */   ld_iX_e     ,
/* 6C */   ld_iX_Ix    ,
/* 6D */   ld_iX_iX    ,
/* 6E */   ld_l_pix    ,
/* 6F */   ld_iX_a     ,
/* 70 */   ld_pix_b    ,
/* 71 */   ld_pix_c    ,
/* 72 */   ld_pix_d    ,
/* 73 */   ld_pix_e    ,
/* 74 */   ld_pix_h    ,
/* 75 */   ld_pix_l    ,
/* 76 */   dummyIX     ,
/* 77 */   ld_pix_a    ,
/* 78 */   dummyIX     ,
/* 79 */   dummyIX     ,
/* 7A */   dummyIX     ,
/* 7B */   dummyIX     ,
/* 7C */   ld_a_Ix     ,
/* 7D */   ld_a_iX     ,
/* 7E */   ld_a_pix    ,
/* 7F */   dummyIX     ,
/* 80 */   dummyIX     ,
/* 81 */   dummyIX     ,
/* 82 */   dummyIX     ,
/* 83 */   dummyIX     ,
/* 84 */   add_a_Ix    ,
/* 85 */   add_a_iX    ,
/* 86 */   add_a_pix   ,
/* 87 */   dummyIX     ,
/* 88 */   dummyIX     ,
/* 89 */   dummyIX     ,
/* 8A */   dummyIX     ,
/* 8B */   dummyIX     ,
/* 8C */   adc_a_Ix    ,
/* 8D */   adc_a_iX    ,
/* 8E */   adc_a_pix   ,
/* 8F */   dummyIX     ,
/* 90 */   dummyIX     ,
/* 91 */   dummyIX     ,
/* 92 */   dummyIX     ,
/* 93 */   dummyIX     ,
/* 94 */   sub_Ix      ,
/* 95 */   sub_iX      ,
/* 96 */   sub_pix     ,
/* 97 */   dummyIX     ,
/* 98 */   dummyIX     ,
/* 99 */   dummyIX     ,
/* 9A */   dummyIX     ,
/* 9B */   dummyIX     ,
/* 9C */   sbc_a_Ix    ,
/* 9D */   sbc_a_iX    ,
/* 9E */   sbc_a_pix   ,
/* 9F */   dummyIX     ,
/* A0 */   dummyIX     ,
/* A1 */   dummyIX     ,
/* A2 */   dummyIX     ,
/* A3 */   dummyIX     ,
/* A4 */   and_Ix      ,
/* A5 */   and_iX      ,
/* A6 */   and_pix     ,
/* A7 */   dummyIX     ,
/* A8 */   dummyIX     ,
/* A9 */   dummyIX     ,
/* AA */   dummyIX     ,
/* AB */   dummyIX     ,
/* AC */   xor_Ix      ,
/* AD */   xor_iX      ,
/* AE */   xor_pix     ,
/* AF */   dummyIX     ,
/* B0 */   dummyIX     ,
/* B1 */   dummyIX     ,
/* B2 */   dummyIX     ,
/* B3 */   dummyIX     ,
/* B4 */   or_Ix       ,
/* B5 */   or_iX       ,
/* B6 */   or_pix      ,
/* B7 */   dummyIX     ,
/* B8 */   dummyIX     ,
/* B9 */   dummyIX     ,
/* BA */   dummyIX     ,
/* BB */   dummyIX     ,
/* BC */   cp_Ix       ,
/* BD */   cp_iX       ,
/* BE */   cp_pix      ,
/* BF */   dummyIX     ,
/* C0 */   dummyIX     ,
/* C1 */   dummyIX     ,
/* C2 */   dummyIX     ,
/* C3 */   dummyIX     ,
/* C4 */   dummyIX     ,
/* C5 */   dummyIX     ,
/* C6 */   dummyIX     ,
/* C7 */   dummyIX     ,
/* C8 */   dummyIX     ,
/* C9 */   dummyIX     ,
/* CA */   dummyIX     ,
/* CB */   execute_IXCB,
/* CC */   dummyIX     ,
/* CD */   dummyIX     ,
/* CE */   dummyIX     ,
/* CF */   dummyIX     ,
/* D0 */   dummyIX     ,
/* D1 */   dummyIX     ,
/* D2 */   dummyIX     ,
/* D3 */   dummyIX     ,
/* D4 */   dummyIX     ,
/* D5 */   dummyIX     ,
/* D6 */   dummyIX     ,
/* D7 */   dummyIX     ,
/* D8 */   dummyIX     ,
/* D9 */   dummyIX     ,
/* DA */   dummyIX     ,
/* DB */   dummyIX     ,
/* DC */   dummyIX     ,
/* DD */   dummyIX     ,
/* DE */   dummyIX     ,
/* DF */   dummyIX     ,
/* E0 */   dummyIX     ,
/* E1 */   pop_ix      ,
/* E2 */   dummyIX     ,
/* E3 */   ex_psp_ix   ,
/* E4 */   dummyIX     ,
/* E5 */   push_ix     ,
/* E6 */   dummyIX     ,
/* E7 */   dummyIX     ,
/* E8 */   dummyIX     ,
/* E9 */   jp_ix       ,
/* EA */   dummyIX     ,
/* EB */   ex_de_ix    ,
/* EC */   dummyIX     ,
/* ED */   dummyIX     ,
/* EE */   dummyIX     ,
/* EF */   dummyIX     ,
/* F0 */   dummyIX     ,
/* F1 */   dummyIX     ,
/* F2 */   dummyIX     ,
/* F3 */   dummyIX     ,
/* F4 */   dummyIX     ,
/* F5 */   dummyIX     ,
/* F6 */   dummyIX     ,
/* F7 */   dummyIX     ,
/* F8 */   dummyIX     ,
/* F9 */   ld_sp_ix    ,
/* FA */   dummyIX     ,
/* FB */   dummyIX     ,
/* FC */   dummyIX     ,
/* FD */   dummyIX     ,
/* FE */   dummyIX     ,
/* FF */   dummyIX
};

static void (*instruc_tablIY[256])() =
{
/* 00 */   dummyIY     ,
/* 01 */   dummyIY     ,
/* 02 */   dummyIY     ,
/* 03 */   dummyIY     ,
/* 04 */   dummyIY     ,
/* 05 */   dummyIY     ,
/* 06 */   dummyIY     ,
/* 07 */   dummyIY     ,
/* 08 */   dummyIY     ,
/* 09 */   add_iy_bc   ,
/* 0A */   dummyIY     ,
/* 0B */   dummyIY     ,
/* 0C */   dummyIY     ,
/* 0D */   dummyIY     ,
/* 0E */   dummyIY     ,
/* 0F */   dummyIY     ,
/* 10 */   dummyIY     ,
/* 11 */   dummyIY     ,
/* 12 */   dummyIY     ,
/* 13 */   dummyIY     ,
/* 14 */   dummyIY     ,
/* 15 */   dummyIY     ,
/* 16 */   dummyIY     ,
/* 17 */   dummyIY     ,
/* 18 */   dummyIY     ,
/* 19 */   add_iy_de   ,
/* 1A */   dummyIY     ,
/* 1B */   dummyIY     ,
/* 1C */   dummyIY     ,
/* 1D */   dummyIY     ,
/* 1E */   dummyIY     ,
/* 1F */   dummyIY     ,
/* 20 */   dummyIY     ,
/* 21 */   ld_iy_nn    ,
/* 22 */   ld_pnn_iy   ,
/* 23 */   inc_iy      ,
/* 24 */   inc_Iy      ,
/* 25 */   dec_Iy      ,
/* 26 */   ld_Iy_n     ,
/* 27 */   dummyIY     ,
/* 28 */   dummyIY     ,
/* 29 */   add_iy_iy   ,
/* 2A */   ld_iy_pnn   ,
/* 2B */   dec_iy      ,
/* 2C */   inc_iY      ,
/* 2D */   dec_iY      ,
/* 2E */   ld_iY_n     ,
/* 2F */   dummyIY     ,
/* 30 */   dummyIY     ,
/* 31 */   dummyIY     ,
/* 32 */   dummyIY     ,
/* 33 */   dummyIY     ,
/* 34 */   inc_piy     ,
/* 35 */   dec_piy     ,
/* 36 */   ld_piy_n    ,
/* 37 */   dummyIY     ,
/* 38 */   dummyIY     ,
/* 39 */   add_iy_sp   ,
/* 3A */   dummyIY     ,
/* 3B */   dummyIY     ,
/* 3C */   dummyIY     ,
/* 3D */   dummyIY     ,
/* 3E */   dummyIY     ,
/* 3F */   dummyIY     ,
/* 40 */   dummyIY     ,
/* 41 */   dummyIY     ,
/* 42 */   dummyIY     ,
/* 43 */   dummyIY     ,
/* 44 */   ld_b_Iy     ,
/* 45 */   ld_b_iY     ,
/* 46 */   ld_b_piy    ,
/* 47 */   dummyIY     ,
/* 48 */   dummyIY     ,
/* 49 */   dummyIY     ,
/* 4A */   dummyIY     ,
/* 4B */   dummyIY     ,
/* 4C */   ld_c_Iy     ,
/* 4D */   ld_c_iY     ,
/* 4E */   ld_c_piy    ,
/* 4F */   dummyIY     ,
/* 50 */   dummyIY     ,
/* 51 */   dummyIY     ,
/* 52 */   dummyIY     ,
/* 53 */   dummyIY     ,
/* 54 */   ld_d_Iy     ,
/* 55 */   ld_d_iY     ,
/* 56 */   ld_d_piy    ,
/* 57 */   dummyIY     ,
/* 58 */   dummyIY     ,
/* 59 */   dummyIY     ,
/* 5A */   dummyIY     ,
/* 5B */   dummyIY     ,
/* 5C */   ld_e_Iy     ,
/* 5D */   ld_e_iY     ,
/* 5E */   ld_e_piy    ,
/* 5F */   dummyIY     ,
/* 60 */   ld_Iy_b     ,
/* 61 */   ld_Iy_c     ,
/* 62 */   ld_Iy_d     ,
/* 63 */   ld_Iy_e     ,
/* 64 */   ld_Iy_Iy    ,
/* 65 */   ld_Iy_iY    ,
/* 66 */   ld_h_piy    ,
/* 67 */   ld_Iy_a     ,
/* 68 */   ld_iY_b     ,
/* 69 */   ld_iY_c     ,
/* 6A */   ld_iY_d     ,
/* 6B */   ld_iY_e     ,
/* 6C */   ld_iY_Iy    ,
/* 6D */   ld_iY_iY    ,
/* 6E */   ld_l_piy    ,
/* 6F */   ld_iY_a     ,
/* 70 */   ld_piy_b    ,
/* 71 */   ld_piy_c    ,
/* 72 */   ld_piy_d    ,
/* 73 */   ld_piy_e    ,
/* 74 */   ld_piy_h    ,
/* 75 */   ld_piy_l    ,
/* 76 */   dummyIY     ,
/* 77 */   ld_piy_a    ,
/* 78 */   dummyIY     ,
/* 79 */   dummyIY     ,
/* 7A */   dummyIY     ,
/* 7B */   dummyIY     ,
/* 7C */   ld_a_Iy     ,
/* 7D */   ld_a_iY     ,
/* 7E */   ld_a_piy    ,
/* 7F */   dummyIY     ,
/* 80 */   dummyIY     ,
/* 81 */   dummyIY     ,
/* 82 */   dummyIY     ,
/* 83 */   dummyIY     ,
/* 84 */   add_a_Iy    ,
/* 85 */   add_a_iY    ,
/* 86 */   add_a_piy   ,
/* 87 */   dummyIY     ,
/* 88 */   dummyIY     ,
/* 89 */   dummyIY     ,
/* 8A */   dummyIY     ,
/* 8B */   dummyIY     ,
/* 8C */   adc_a_Iy    ,
/* 8D */   adc_a_iY    ,
/* 8E */   adc_a_piy   ,
/* 8F */   dummyIY     ,
/* 90 */   dummyIY     ,
/* 91 */   dummyIY     ,
/* 92 */   dummyIY     ,
/* 93 */   dummyIY     ,
/* 94 */   sub_Iy      ,
/* 95 */   sub_iY      ,
/* 96 */   sub_piy     ,
/* 97 */   dummyIY     ,
/* 98 */   dummyIY     ,
/* 99 */   dummyIY     ,
/* 9A */   dummyIY     ,
/* 9B */   dummyIY     ,
/* 9C */   sbc_a_Iy    ,
/* 9D */   sbc_a_iY    ,
/* 9E */   sbc_a_piy   ,
/* 9F */   dummyIY     ,
/* A0 */   dummyIY     ,
/* A1 */   dummyIY     ,
/* A2 */   dummyIY     ,
/* A3 */   dummyIY     ,
/* A4 */   and_Iy      ,
/* A5 */   and_iY      ,
/* A6 */   and_piy     ,
/* A7 */   dummyIY     ,
/* A8 */   dummyIY     ,
/* A9 */   dummyIY     ,
/* AA */   dummyIY     ,
/* AB */   dummyIY     ,
/* AC */   xor_Iy      ,
/* AD */   xor_iY      ,
/* AE */   xor_piy     ,
/* AF */   dummyIY     ,
/* B0 */   dummyIY     ,
/* B1 */   dummyIY     ,
/* B2 */   dummyIY     ,
/* B3 */   dummyIY     ,
/* B4 */   or_Iy       ,
/* B5 */   or_iY       ,
/* B6 */   or_piy      ,
/* B7 */   dummyIY     ,
/* B8 */   dummyIY     ,
/* B9 */   dummyIY     ,
/* BA */   dummyIY     ,
/* BB */   dummyIY     ,
/* BC */   cp_Iy       ,
/* BD */   cp_iY       ,
/* BE */   cp_piy      ,
/* BF */   dummyIY     ,
/* C0 */   dummyIY     ,
/* C1 */   dummyIY     ,
/* C2 */   dummyIY     ,
/* C3 */   dummyIY     ,
/* C4 */   dummyIY     ,
/* C5 */   dummyIY     ,
/* C6 */   dummyIY     ,
/* C7 */   dummyIY     ,
/* C8 */   dummyIY     ,
/* C9 */   dummyIY     ,
/* CA */   dummyIY     ,
/* CB */   execute_IYCB,
/* CC */   dummyIY     ,
/* CD */   dummyIY     ,
/* CE */   dummyIY     ,
/* CF */   dummyIY     ,
/* D0 */   dummyIY     ,
/* D1 */   dummyIY     ,
/* D2 */   dummyIY     ,
/* D3 */   dummyIY     ,
/* D4 */   dummyIY     ,
/* D5 */   dummyIY     ,
/* D6 */   dummyIY     ,
/* D7 */   dummyIY     ,
/* D8 */   dummyIY     ,
/* D9 */   dummyIY     ,
/* DA */   dummyIY     ,
/* DB */   dummyIY     ,
/* DC */   dummyIY     ,
/* DD */   dummyIY     ,
/* DE */   dummyIY     ,
/* DF */   dummyIY     ,
/* E0 */   dummyIY     ,
/* E1 */   pop_iy      ,
/* E2 */   dummyIY     ,
/* E3 */   ex_psp_iy   ,
/* E4 */   dummyIY     ,
/* E5 */   push_iy     ,
/* E6 */   dummyIY     ,
/* E7 */   dummyIY     ,
/* E8 */   dummyIY     ,
/* E9 */   jp_iy       ,
/* EA */   dummyIY     ,
/* EB */   ex_de_iy    ,
/* EC */   dummyIY     ,
/* ED */   dummyIY     ,
/* EE */   dummyIY     ,
/* EF */   dummyIY     ,
/* F0 */   dummyIY     ,
/* F1 */   dummyIY     ,
/* F2 */   dummyIY     ,
/* F3 */   dummyIY     ,
/* F4 */   dummyIY     ,
/* F5 */   dummyIY     ,
/* F6 */   dummyIY     ,
/* F7 */   dummyIY     ,
/* F8 */   dummyIY     ,
/* F9 */   ld_sp_iy    ,
/* FA */   dummyIY     ,
/* FB */   dummyIY     ,
/* FC */   dummyIY     ,
/* FD */   dummyIY     ,
/* FE */   dummyIY     ,
/* FF */   dummyIY
};

static void (*instruc_tablCB[256])() =
{
/* 00 */   rlc_b       ,
/* 01 */   rlc_c       ,
/* 02 */   rlc_d       ,
/* 03 */   rlc_e       ,
/* 04 */   rlc_h       ,
/* 05 */   rlc_l       ,
/* 06 */   rlc_phl     ,
/* 07 */   rlc_a       ,
/* 08 */   rrc_b       ,
/* 09 */   rrc_c       ,
/* 0A */   rrc_d       ,
/* 0B */   rrc_e       ,
/* 0C */   rrc_h       ,
/* 0D */   rrc_l       ,
/* 0E */   rrc_phl     ,
/* 0F */   rrc_a       ,
/* 10 */   rl_b        ,
/* 11 */   rl_c        ,
/* 12 */   rl_d        ,
/* 13 */   rl_e        ,
/* 14 */   rl_h        ,
/* 15 */   rl_l        ,
/* 16 */   rl_phl      ,
/* 17 */   rl_a        ,
/* 18 */   rr_b        ,
/* 19 */   rr_c        ,
/* 1A */   rr_d        ,
/* 1B */   rr_e        ,
/* 1C */   rr_h        ,
/* 1D */   rr_l        ,
/* 1E */   rr_phl      ,
/* 1F */   rr_a        ,
/* 20 */   sla_b       ,
/* 21 */   sla_c       ,
/* 22 */   sla_d       ,
/* 23 */   sla_e       ,
/* 24 */   sla_h       ,
/* 25 */   sla_l       ,
/* 26 */   sla_phl     ,
/* 27 */   sla_a       ,
/* 28 */   sra_b       ,
/* 29 */   sra_c       ,
/* 2A */   sra_d       ,
/* 2B */   sra_e       ,
/* 2C */   sra_h       ,
/* 2D */   sra_l       ,
/* 2E */   sra_phl     ,
/* 2F */   sra_a       ,
/* 30 */   sll_b       ,
/* 31 */   sll_c       ,
/* 32 */   sll_d       ,
/* 33 */   sll_e       ,
/* 34 */   sll_h       ,
/* 35 */   sll_l       ,
/* 36 */   sll_phl     ,
/* 37 */   sll_a       ,
/* 38 */   srl_b       ,
/* 39 */   srl_c       ,
/* 3A */   srl_d       ,
/* 3B */   srl_e       ,
/* 3C */   srl_h       ,
/* 3D */   srl_l       ,
/* 3E */   srl_phl     ,
/* 3F */   srl_a       ,
/* 40 */   bit_0_b     ,
/* 41 */   bit_0_c     ,
/* 42 */   bit_0_d     ,
/* 43 */   bit_0_e     ,
/* 44 */   bit_0_h     ,
/* 45 */   bit_0_l     ,
/* 46 */   bit_0_phl   ,
/* 47 */   bit_0_a     ,
/* 48 */   bit_1_b     ,
/* 49 */   bit_1_c     ,
/* 4A */   bit_1_d     ,
/* 4B */   bit_1_e     ,
/* 4C */   bit_1_h     ,
/* 4D */   bit_1_l     ,
/* 4E */   bit_1_phl   ,
/* 4F */   bit_1_a     ,
/* 50 */   bit_2_b     ,
/* 51 */   bit_2_c     ,
/* 52 */   bit_2_d     ,
/* 53 */   bit_2_e     ,
/* 54 */   bit_2_h     ,
/* 55 */   bit_2_l     ,
/* 56 */   bit_2_phl   ,
/* 57 */   bit_2_a     ,
/* 58 */   bit_3_b     ,
/* 59 */   bit_3_c     ,
/* 5A */   bit_3_d     ,
/* 5B */   bit_3_e     ,
/* 5C */   bit_3_h     ,
/* 5D */   bit_3_l     ,
/* 5E */   bit_3_phl   ,
/* 5F */   bit_3_a     ,
/* 60 */   bit_4_b     ,
/* 61 */   bit_4_c     ,
/* 62 */   bit_4_d     ,
/* 63 */   bit_4_e     ,
/* 64 */   bit_4_h     ,
/* 65 */   bit_4_l     ,
/* 66 */   bit_4_phl   ,
/* 67 */   bit_4_a     ,
/* 68 */   bit_5_b     ,
/* 69 */   bit_5_c     ,
/* 6A */   bit_5_d     ,
/* 6B */   bit_5_e     ,
/* 6C */   bit_5_h     ,
/* 6D */   bit_5_l     ,
/* 6E */   bit_5_phl   ,
/* 6F */   bit_5_a     ,
/* 70 */   bit_6_b     ,
/* 71 */   bit_6_c     ,
/* 72 */   bit_6_d     ,
/* 73 */   bit_6_e     ,
/* 74 */   bit_6_h     ,
/* 75 */   bit_6_l     ,
/* 76 */   bit_6_phl   ,
/* 77 */   bit_6_a     ,
/* 78 */   bit_7_b     ,
/* 79 */   bit_7_c     ,
/* 7A */   bit_7_d     ,
/* 7B */   bit_7_e     ,
/* 7C */   bit_7_h     ,
/* 7D */   bit_7_l     ,
/* 7E */   bit_7_phl   ,
/* 7F */   bit_7_a     ,
/* 80 */   res_0_b     ,
/* 81 */   res_0_c     ,
/* 82 */   res_0_d     ,
/* 83 */   res_0_e     ,
/* 84 */   res_0_h     ,
/* 85 */   res_0_l     ,
/* 86 */   res_0_phl   ,
/* 87 */   res_0_a     ,
/* 88 */   res_1_b     ,
/* 89 */   res_1_c     ,
/* 8A */   res_1_d     ,
/* 8B */   res_1_e     ,
/* 8C */   res_1_h     ,
/* 8D */   res_1_l     ,
/* 8E */   res_1_phl   ,
/* 8F */   res_1_a     ,
/* 90 */   res_2_b     ,
/* 91 */   res_2_c     ,
/* 92 */   res_2_d     ,
/* 93 */   res_2_e     ,
/* 94 */   res_2_h     ,
/* 95 */   res_2_l     ,
/* 96 */   res_2_phl   ,
/* 97 */   res_2_a     ,
/* 98 */   res_3_b     ,
/* 99 */   res_3_c     ,
/* 9A */   res_3_d     ,
/* 9B */   res_3_e     ,
/* 9C */   res_3_h     ,
/* 9D */   res_3_l     ,
/* 9E */   res_3_phl   ,
/* 9F */   res_3_a     ,
/* A0 */   res_4_b     ,
/* A1 */   res_4_c     ,
/* A2 */   res_4_d     ,
/* A3 */   res_4_e     ,
/* A4 */   res_4_h     ,
/* A5 */   res_4_l     ,
/* A6 */   res_4_phl   ,
/* A7 */   res_4_a     ,
/* A8 */   res_5_b     ,
/* A9 */   res_5_c     ,
/* AA */   res_5_d     ,
/* AB */   res_5_e     ,
/* AC */   res_5_h     ,
/* AD */   res_5_l     ,
/* AE */   res_5_phl   ,
/* AF */   res_5_a     ,
/* B0 */   res_6_b     ,
/* B1 */   res_6_c     ,
/* B2 */   res_6_d     ,
/* B3 */   res_6_e     ,
/* B4 */   res_6_h     ,
/* B5 */   res_6_l     ,
/* B6 */   res_6_phl   ,
/* B7 */   res_6_a     ,
/* B8 */   res_7_b     ,
/* B9 */   res_7_c     ,
/* BA */   res_7_d     ,
/* BB */   res_7_e     ,
/* BC */   res_7_h     ,
/* BD */   res_7_l     ,
/* BE */   res_7_phl   ,
/* BF */   res_7_a     ,
/* C0 */   set_0_b     ,
/* C1 */   set_0_c     ,
/* C2 */   set_0_d     ,
/* C3 */   set_0_e     ,
/* C4 */   set_0_h     ,
/* C5 */   set_0_l     ,
/* C6 */   set_0_phl   ,
/* C7 */   set_0_a     ,
/* C8 */   set_1_b     ,
/* C9 */   set_1_c     ,
/* CA */   set_1_d     ,
/* CB */   set_1_e     ,
/* CC */   set_1_h     ,
/* CD */   set_1_l     ,
/* CE */   set_1_phl   ,
/* CF */   set_1_a     ,
/* D0 */   set_2_b     ,
/* D1 */   set_2_c     ,
/* D2 */   set_2_d     ,
/* D3 */   set_2_e     ,
/* D4 */   set_2_h     ,
/* D5 */   set_2_l     ,
/* D6 */   set_2_phl   ,
/* D7 */   set_2_a     ,
/* D8 */   set_3_b     ,
/* D9 */   set_3_c     ,
/* DA */   set_3_d     ,
/* DB */   set_3_e     ,
/* DC */   set_3_h     ,
/* DD */   set_3_l     ,
/* DE */   set_3_phl   ,
/* DF */   set_3_a     ,
/* E0 */   set_4_b     ,
/* E1 */   set_4_c     ,
/* E2 */   set_4_d     ,
/* E3 */   set_4_e     ,
/* E4 */   set_4_h     ,
/* E5 */   set_4_l     ,
/* E6 */   set_4_phl   ,
/* E7 */   set_4_a     ,
/* E8 */   set_5_b     ,
/* E9 */   set_5_c     ,
/* EA */   set_5_d     ,
/* EB */   set_5_e     ,
/* EC */   set_5_h     ,
/* ED */   set_5_l     ,
/* EE */   set_5_phl   ,
/* EF */   set_5_a     ,
/* F0 */   set_6_b     ,
/* F1 */   set_6_c     ,
/* F2 */   set_6_d     ,
/* F3 */   set_6_e     ,
/* F4 */   set_6_h     ,
/* F5 */   set_6_l     ,
/* F6 */   set_6_phl   ,
/* F7 */   set_6_a     ,
/* F8 */   set_7_b     ,
/* F9 */   set_7_c     ,
/* FA */   set_7_d     ,
/* FB */   set_7_e     ,
/* FC */   set_7_h     ,
/* FD */   set_7_l     ,
/* FE */   set_7_phl   ,
/* FF */   set_7_a
};

static void (*instruc_tablIXCB[256])() =
{
/* 00 */   rlc_pixb    ,
/* 01 */   rlc_pixc    ,
/* 02 */   rlc_pixd    ,
/* 03 */   rlc_pixe    ,
/* 04 */   rlc_Ix      ,  /* ?? */
/* 05 */   rlc_iX      ,  /* ?? */
/* 06 */   rlc_pix     ,
/* 07 */   rlc_pixa    ,
/* 08 */   rrc_pixb    ,
/* 09 */   rrc_pixc    ,
/* 0A */   rrc_pixd    ,
/* 0B */   rrc_pixe    ,
/* 0C */   rrc_Ix      ,  /* ?? */
/* 0D */   rrc_iX      ,  /* ?? */
/* 0E */   rrc_pix     ,
/* 0F */   rrc_pixa    ,
/* 10 */   rl_pixb     ,
/* 11 */   rl_pixc     ,
/* 12 */   rl_pixd     ,
/* 13 */   rl_pixe     ,
/* 14 */   rl_Ix       ,  /* ?? */
/* 15 */   rl_iX       ,  /* ?? */
/* 16 */   rl_pix      ,
/* 17 */   rl_pixa     ,
/* 18 */   rr_pixb     ,
/* 19 */   rr_pixc     ,
/* 1A */   rr_pixd     ,
/* 1B */   rr_pixe     ,
/* 1C */   rr_Ix       ,  /* ?? */
/* 1D */   rr_iX       ,  /* ?? */
/* 1E */   rr_pix      ,
/* 1F */   rr_pixa     ,
/* 20 */   sla_pixb    ,
/* 21 */   sla_pixc    ,
/* 22 */   sla_pixd    ,
/* 23 */   sla_pixe    ,
/* 24 */   sla_Ix      ,  /* ?? */
/* 25 */   sla_iX      ,  /* ?? */
/* 26 */   sla_pix     ,
/* 27 */   sla_pixa    ,
/* 28 */   sra_pixb    ,
/* 29 */   sra_pixc    ,
/* 2A */   sra_pixd    ,
/* 2B */   sra_pixe    ,
/* 2C */   sra_Ix      ,  /* ?? */
/* 2D */   sra_iX      ,  /* ?? */
/* 2E */   sra_pix     ,
/* 2F */   sra_pixa    ,
/* 30 */   sll_pixb    ,
/* 31 */   sll_pixc    ,
/* 32 */   sll_pixd    ,
/* 33 */   sll_pixe    ,
/* 34 */   sll_Ix      ,  /* ?? */
/* 35 */   sll_iX      ,  /* ?? */
/* 36 */   sll_pix     ,
/* 37 */   sll_pixa    ,
/* 38 */   srl_pixb    ,
/* 39 */   srl_pixc    ,
/* 3A */   srl_pixd    ,
/* 3B */   srl_pixe    ,
/* 3C */   srl_Ix      ,  /* ?? */
/* 3D */   srl_iX      ,  /* ?? */
/* 3E */   srl_pix     ,
/* 3F */   srl_pixa    ,
/* 40 */   bit_0_pix   ,
/* 41 */   bit_0_pix   ,
/* 42 */   bit_0_pix   ,
/* 43 */   bit_0_pix   ,
/* 44 */   bit_0_Ix    ,	/* ?? */
/* 45 */   bit_0_iX    ,	/* ?? */
/* 46 */   bit_0_pix   ,
/* 47 */   bit_0_pix   ,
/* 48 */   bit_1_pix   ,
/* 49 */   bit_1_pix   ,
/* 4A */   bit_1_pix   ,
/* 4B */   bit_1_pix   ,
/* 4C */   bit_1_Ix    ,	/* ?? */
/* 4D */   bit_1_iX    ,  /* ?? */
/* 4E */   bit_1_pix   ,
/* 4F */   bit_1_pix   ,
/* 50 */   bit_2_pix   ,
/* 51 */   bit_2_pix   ,
/* 52 */   bit_2_pix   ,
/* 53 */   bit_2_pix   ,
/* 54 */   bit_2_Ix    ,  /* ?? */
/* 55 */   bit_2_iX    ,  /* ?? */
/* 56 */   bit_2_pix   ,
/* 57 */   bit_2_pix   ,
/* 58 */   bit_3_pix   ,
/* 59 */   bit_3_pix   ,
/* 5A */   bit_3_pix   ,
/* 5B */   bit_3_pix   ,
/* 5C */   bit_3_Ix    ,  /* ?? */
/* 5D */   bit_3_iX    ,  /* ?? */
/* 5E */   bit_3_pix   ,
/* 5F */   bit_3_pix   ,
/* 60 */   bit_4_pix   ,
/* 61 */   bit_4_pix   ,
/* 62 */   bit_4_pix   ,
/* 63 */   bit_4_pix   ,
/* 64 */   bit_4_Ix    ,  /* ?? */
/* 65 */   bit_4_iX    ,  /* ?? */
/* 66 */   bit_4_pix   ,
/* 67 */   bit_4_pix   ,
/* 68 */   bit_5_pix   ,
/* 69 */   bit_5_pix   ,
/* 6A */   bit_5_pix   ,
/* 6B */   bit_5_pix   ,
/* 6C */   bit_5_Ix    ,  /* ?? */
/* 6D */   bit_5_iX    ,  /* ?? */
/* 6E */   bit_5_pix   ,
/* 6F */   bit_5_pix   ,
/* 70 */   bit_6_pix   ,
/* 71 */   bit_6_pix   ,
/* 72 */   bit_6_pix   ,
/* 73 */   bit_6_pix   ,
/* 74 */   bit_6_Ix    ,  /* ?? */
/* 75 */   bit_6_iX    ,  /* ?? */
/* 76 */   bit_6_pix   ,
/* 77 */   bit_6_pix   ,
/* 78 */   bit_7_pix   ,
/* 79 */   bit_7_pix   ,
/* 7A */   bit_7_pix   ,
/* 7B */   bit_7_pix   ,
/* 7C */   bit_7_Ix    ,  /* ?? */
/* 7D */   bit_7_iX    ,  /* ?? */
/* 7E */   bit_7_pix   ,
/* 7F */   bit_7_pix   ,
/* 80 */   res_0_pixb  ,
/* 81 */   res_0_pixc  ,
/* 82 */   res_0_pixd  ,
/* 83 */   res_0_pixe  ,
/* 84 */   res_0_Ix    ,  /* ?? */
/* 85 */   res_0_iX    ,  /* ?? */
/* 86 */   res_0_pix   ,
/* 87 */   res_0_pixa  ,
/* 88 */   res_1_pixb  ,
/* 89 */   res_1_pixc  ,
/* 8A */   res_1_pixd  ,
/* 8B */   res_1_pixe  ,
/* 8C */   res_1_Ix    ,  /* ?? */
/* 8D */   res_1_iX    ,  /* ?? */
/* 8E */   res_1_pix   ,
/* 8F */   res_1_pixa  ,
/* 90 */   res_2_pixb  ,
/* 91 */   res_2_pixc  ,
/* 92 */   res_2_pixd  ,
/* 93 */   res_2_pixe  ,
/* 94 */   res_2_Ix    ,  /* ?? */
/* 95 */   res_2_iX    ,  /* ?? */
/* 96 */   res_2_pix   ,
/* 97 */   res_2_pixa  ,
/* 98 */   res_3_pixb  ,
/* 99 */   res_3_pixc  ,
/* 9A */   res_3_pixd  ,
/* 9B */   res_3_pixe  ,
/* 9C */   res_3_Ix    ,  /* ?? */
/* 9D */   res_3_iX    ,  /* ?? */
/* 9E */   res_3_pix   ,
/* 9F */   res_3_pixa  ,
/* A0 */   res_4_pixb  ,
/* A1 */   res_4_pixc  ,
/* A2 */   res_4_pixd  ,
/* A3 */   res_4_pixe  ,
/* A4 */   res_4_Ix    ,  /* ?? */
/* A5 */   res_4_iX    ,  /* ?? */
/* A6 */   res_4_pix   ,
/* A7 */   res_4_pixa  ,
/* A8 */   res_5_pixb  ,
/* A9 */   res_5_pixc  ,
/* AA */   res_5_pixd  ,
/* AB */   res_5_pixe  ,
/* AC */   res_5_Ix    ,  /* ?? */
/* AD */   res_5_iX    ,  /* ?? */
/* AE */   res_5_pix   ,
/* AF */   res_5_pixa  ,
/* B0 */   res_6_pixb  ,
/* B1 */   res_6_pixc  ,
/* B2 */   res_6_pixd  ,
/* B3 */   res_6_pixe  ,
/* B4 */   res_6_Ix    ,  /* ?? */
/* B5 */   res_6_iX    ,  /* ?? */
/* B6 */   res_6_pix   ,
/* B7 */   res_6_pixa  ,
/* B8 */   res_7_pixb  ,
/* B9 */   res_7_pixc  ,
/* BA */   res_7_pixd  ,
/* BB */   res_7_pixe  ,
/* BC */   res_7_Ix    ,  /* ?? */
/* BD */   res_7_iX    ,  /* ?? */
/* BE */   res_7_pix   ,
/* BF */   res_7_pixa  ,
/* C0 */   set_0_pixb  ,
/* C1 */   set_0_pixc  ,
/* C2 */   set_0_pixd  ,
/* C3 */   set_0_pixe  ,
/* C4 */   set_0_Ix    ,  /* ?? */
/* C5 */   set_0_iX    ,  /* ?? */
/* C6 */   set_0_pix   ,
/* C7 */   set_0_pixa  ,
/* C8 */   set_1_pixb  ,
/* C9 */   set_1_pixc  ,
/* CA */   set_1_pixd  ,
/* CB */   set_1_pixe  ,
/* CC */   set_1_Ix    ,  /* ?? */
/* CD */   set_1_iX    ,  /* ?? */
/* CE */   set_1_pix   ,
/* CF */   set_1_pixa  ,
/* D0 */   set_2_pixb  ,
/* D1 */   set_2_pixc  ,
/* D2 */   set_2_pixd  ,
/* D3 */   set_2_pixe  ,
/* D4 */   set_2_Ix    ,  /* ?? */
/* D5 */   set_2_iX    ,  /* ?? */
/* D6 */   set_2_pix   ,
/* D7 */   set_2_pixa  ,
/* D8 */   set_3_pixb  ,
/* D9 */   set_3_pixc  ,
/* DA */   set_3_pixd  ,
/* DB */   set_3_pixe  ,
/* DC */   set_3_Ix    ,  /* ?? */
/* DD */   set_3_iX    ,  /* ?? */
/* DE */   set_3_pix   ,
/* DF */   set_3_pixa  ,
/* E0 */   set_4_pixb  ,
/* E1 */   set_4_pixc  ,
/* E2 */   set_4_pixd  ,
/* E3 */   set_4_pixe  ,
/* E4 */   set_4_Ix    ,
/* E5 */   set_4_iX    ,
/* E6 */   set_4_pix   ,
/* E7 */   set_4_pixa  ,
/* E8 */   set_5_pixb  ,
/* E9 */   set_5_pixc  ,
/* EA */   set_5_pixd  ,
/* EB */   set_5_pixe  ,
/* EC */   set_5_Ix    ,  /* ?? */
/* ED */   set_5_iX    ,  /* ?? */
/* EE */   set_5_pix   ,
/* EF */   set_5_pixa  ,
/* F0 */   set_6_pixb  ,
/* F1 */   set_6_pixc  ,
/* F2 */   set_6_pixd  ,
/* F3 */   set_6_pixe  ,
/* F4 */   set_6_Ix    ,  /* ?? */
/* F5 */   set_6_iX    ,  /* ?? */
/* F6 */   set_6_pix   ,
/* F7 */   set_6_pixa  ,
/* F8 */   set_7_pixb  ,
/* F9 */   set_7_pixc  ,
/* FA */   set_7_pixd  ,
/* FB */   set_7_pixe  ,
/* FC */   set_7_Ix    ,  /* ?? */
/* FD */   set_7_iX    ,  /* ?? */
/* FE */   set_7_pix   ,
/* FF */   set_7_pixa
};

static void (*instruc_tablIYCB[256])() =
{
/* 00 */   rlc_piyb    ,
/* 01 */   rlc_piyc    ,
/* 02 */   rlc_piyd    ,
/* 03 */   rlc_piye    ,
/* 04 */   rlc_Iy      ,  /* ?? */
/* 05 */   rlc_iY      ,  /* ?? */
/* 06 */   rlc_piy     ,
/* 07 */   rlc_piya    ,
/* 08 */   rrc_piyb    ,
/* 09 */   rrc_piyc    ,
/* 0A */   rrc_piyd    ,
/* 0B */   rrc_piye    ,
/* 0C */   rrc_Iy      ,  /* ?? */
/* 0D */   rrc_iY      ,  /* ?? */
/* 0E */   rrc_piy     ,
/* 0F */   rrc_piya    ,
/* 10 */   rl_piyb     ,
/* 11 */   rl_piyc     ,
/* 12 */   rl_piyd     ,
/* 13 */   rl_piye     ,
/* 14 */   rl_Iy       ,  /* ?? */
/* 15 */   rl_iY       ,  /* ?? */
/* 16 */   rl_piy      ,
/* 17 */   rl_piya     ,
/* 18 */   rr_piyb     ,
/* 19 */   rr_piyc     ,
/* 1A */   rr_piyd     ,
/* 1B */   rr_piye     ,
/* 1C */   rr_Iy       ,  /* ?? */
/* 1D */   rr_iY       ,  /* ?? */
/* 1E */   rr_piy      ,
/* 1F */   rr_piya     ,
/* 20 */   sla_piyb    ,
/* 21 */   sla_piyc    ,
/* 22 */   sla_piyd    ,
/* 23 */   sla_piye    ,
/* 24 */   sla_Iy      ,  /* ?? */
/* 25 */   sla_iY      ,  /* ?? */
/* 26 */   sla_piy     ,
/* 27 */   sla_piya    ,
/* 28 */   sra_piyb    ,
/* 29 */   sra_piyc    ,
/* 2A */   sra_piyd    ,
/* 2B */   sra_piye    ,
/* 2C */   sra_Iy      ,  /* ?? */
/* 2D */   sra_iY      ,  /* ?? */
/* 2E */   sra_piy     ,
/* 2F */   sra_piya    ,
/* 30 */   sll_piyb    ,
/* 31 */   sll_piyc    ,
/* 32 */   sll_piyd    ,
/* 33 */   sll_piye    ,
/* 34 */   sll_Iy      ,  /* ?? */
/* 35 */   sll_iY      ,  /* ?? */
/* 36 */   sll_piy     ,
/* 37 */   sll_piya    ,
/* 38 */   srl_piyb    ,
/* 39 */   srl_piyc    ,
/* 3A */   srl_piyd    ,
/* 3B */   srl_piye    ,
/* 3C */   srl_Iy      ,  /* ?? */
/* 3D */   srl_iY      ,  /* ?? */
/* 3E */   srl_piy     ,
/* 3F */   srl_piya    ,
/* 40 */   bit_0_piy   ,
/* 41 */   bit_0_piy   ,
/* 42 */   bit_0_piy   ,
/* 43 */   bit_0_piy   ,
/* 44 */   bit_0_Iy    ,  /* ?? */
/* 45 */   bit_0_iY    ,  /* ?? */
/* 46 */   bit_0_piy   ,
/* 47 */   bit_0_piy   ,
/* 48 */   bit_1_piy   ,
/* 49 */   bit_1_piy   ,
/* 4A */   bit_1_piy   ,
/* 4B */   bit_1_piy   ,
/* 4C */   bit_1_Iy    ,  /* ?? */
/* 4D */   bit_1_iY    ,  /* ?? */
/* 4E */   bit_1_piy   ,
/* 4F */   bit_1_piy   ,
/* 50 */   bit_2_piy   ,
/* 51 */   bit_2_piy   ,
/* 52 */   bit_2_piy   ,
/* 53 */   bit_2_piy   ,
/* 54 */   bit_2_Iy    ,  /* ?? */
/* 55 */   bit_2_iY    ,  /* ?? */
/* 56 */   bit_2_piy   ,
/* 57 */   bit_2_piy   ,
/* 58 */   bit_3_piy   ,
/* 59 */   bit_3_piy   ,
/* 5A */   bit_3_piy   ,
/* 5B */   bit_3_piy   ,
/* 5C */   bit_3_Iy    ,  /* ?? */
/* 5D */   bit_3_iY    ,  /* ?? */
/* 5E */   bit_3_piy   ,
/* 5F */   bit_3_piy   ,
/* 60 */   bit_4_piy   ,
/* 61 */   bit_4_piy   ,
/* 62 */   bit_4_piy   ,
/* 63 */   bit_4_piy   ,
/* 64 */   bit_4_Iy    ,  /* ?? */
/* 65 */   bit_4_iY    ,  /* ?? */
/* 66 */   bit_4_piy   ,
/* 67 */   bit_4_piy   ,
/* 68 */   bit_5_piy   ,
/* 69 */   bit_5_piy   ,
/* 6A */   bit_5_piy   ,
/* 6B */   bit_5_piy   ,
/* 6C */   bit_5_Iy    ,  /* ?? */
/* 6D */   bit_5_iY    ,  /* ?? */
/* 6E */   bit_5_piy   ,
/* 6F */   bit_5_piy   ,
/* 70 */   bit_6_piy   ,
/* 71 */   bit_6_piy   ,
/* 72 */   bit_6_piy   ,
/* 73 */   bit_6_piy   ,
/* 74 */   bit_6_Iy    ,  /* ?? */
/* 75 */   bit_6_iY    ,  /* ?? */
/* 76 */   bit_6_piy   ,
/* 77 */   bit_6_piy   ,
/* 78 */   bit_7_piy   ,
/* 79 */   bit_7_piy   ,
/* 7A */   bit_7_piy   ,
/* 7B */   bit_7_piy   ,
/* 7C */   bit_7_Iy    ,  /* ?? */
/* 7D */   bit_7_iY    ,  /* ?? */
/* 7E */   bit_7_piy   ,
/* 7F */   bit_7_piy   ,
/* 80 */   res_0_piyb  ,
/* 81 */   res_0_piyc  ,
/* 82 */   res_0_piyd  ,
/* 83 */   res_0_piye  ,
/* 84 */   res_0_Iy    ,  /* ?? */
/* 85 */   res_0_iY    ,  /* ?? */
/* 86 */   res_0_piy   ,
/* 87 */   res_0_piya  ,
/* 88 */   res_1_piyb  ,
/* 89 */   res_1_piyc  ,
/* 8A */   res_1_piyd  ,
/* 8B */   res_1_piye  ,
/* 8C */   res_1_Iy    ,  /* ?? */
/* 8D */   res_1_iY    ,  /* ?? */
/* 8E */   res_1_piy   ,
/* 8F */   res_1_piya  ,
/* 90 */   res_2_piyb  ,
/* 91 */   res_2_piyc  ,
/* 92 */   res_2_piyd  ,
/* 93 */   res_2_piye  ,
/* 94 */   res_2_Iy    ,  /* ?? */
/* 95 */   res_2_iY    ,  /* ?? */
/* 96 */   res_2_piy   ,
/* 97 */   res_2_piya  ,
/* 98 */   res_3_piyb  ,
/* 99 */   res_3_piyc  ,
/* 9A */   res_3_piyd  ,
/* 9B */   res_3_piye  ,
/* 9C */   res_3_Iy    ,  /* ?? */
/* 9D */   res_3_iY    ,  /* ?? */
/* 9E */   res_3_piy   ,
/* 9F */   res_3_piya  ,
/* A0 */   res_4_piyb  ,
/* A1 */   res_4_piyc  ,
/* A2 */   res_4_piyd  ,
/* A3 */   res_4_piye  ,
/* A4 */   res_4_Iy    ,  /* ?? */
/* A5 */   res_4_iY    ,  /* ?? */
/* A6 */   res_4_piy   ,
/* A7 */   res_4_piya  ,
/* A8 */   res_5_piyb  ,
/* A9 */   res_5_piyc  ,
/* AA */   res_5_piyd  ,
/* AB */   res_5_piye  ,
/* AC */   res_5_Iy    ,  /* ?? */
/* AD */   res_5_iY    ,  /* ?? */
/* AE */   res_5_piy   ,
/* AF */   res_5_piya  ,
/* B0 */   res_6_piyb  ,
/* B1 */   res_6_piyc  ,
/* B2 */   res_6_piyd  ,
/* B3 */   res_6_piye  ,
/* B4 */   res_6_Iy    ,  /* ?? */
/* B5 */   res_6_iY    ,  /* ?? */
/* B6 */   res_6_piy   ,
/* B7 */   res_6_piya  ,
/* B8 */   res_7_piyb  ,
/* B9 */   res_7_piyc  ,
/* BA */   res_7_piyd  ,
/* BB */   res_7_piye  ,
/* BC */   res_7_Iy    ,  /* ?? */
/* BD */   res_7_iY    ,  /* ?? */
/* BE */   res_7_piy   ,
/* BF */   res_7_piya  ,
/* C0 */   set_0_piyb  ,
/* C1 */   set_0_piyc  ,
/* C2 */   set_0_piyd  ,
/* C3 */   set_0_piye  ,
/* C4 */   set_0_Iy    ,  /* ?? */
/* C5 */   set_0_iY    ,  /* ?? */
/* C6 */   set_0_piy   ,
/* C7 */   set_0_piya  ,
/* C8 */   set_1_piyb  ,
/* C9 */   set_1_piyc  ,
/* CA */   set_1_piyd  ,
/* CB */   set_1_piye  ,
/* CC */   set_1_Iy    ,  /* ?? */
/* CD */   set_1_iY    ,  /* ?? */
/* CE */   set_1_piy   ,
/* CF */   set_1_piya  ,
/* D0 */   set_2_piyb  ,
/* D1 */   set_2_piyc  ,
/* D2 */   set_2_piyd  ,
/* D3 */   set_2_piye  ,
/* D4 */   set_2_Iy    ,  /* ?? */
/* D5 */   set_2_iY    ,  /* ?? */
/* D6 */   set_2_piy   ,
/* D7 */   set_2_piya  ,
/* D8 */   set_3_piyb  ,
/* D9 */   set_3_piyc  ,
/* DA */   set_3_piyd  ,
/* DB */   set_3_piye  ,
/* DC */   set_3_Iy    ,  /* ?? */
/* DD */   set_3_iY    ,  /* ?? */
/* DE */   set_3_piy   ,
/* DF */   set_3_piya  ,
/* E0 */   set_4_piyb  ,
/* E1 */   set_4_piyc  ,
/* E2 */   set_4_piyd  ,
/* E3 */   set_4_piye  ,
/* E4 */   set_4_Iy    ,
/* E5 */   set_4_iY    ,
/* E6 */   set_4_piy   ,
/* E7 */   set_4_piya  ,
/* E8 */   set_5_piyb  ,
/* E9 */   set_5_piyc  ,
/* EA */   set_5_piyd  ,
/* EB */   set_5_piye  ,
/* EC */   set_5_Iy    ,  /* ?? */
/* ED */   set_5_iY    ,  /* ?? */
/* EE */   set_5_piy   ,
/* EF */   set_5_piya  ,
/* F0 */   set_6_piyb  ,
/* F1 */   set_6_piyc  ,
/* F2 */   set_6_piyd  ,
/* F3 */   set_6_piye  ,
/* F4 */   set_6_Iy    ,  /* ?? */
/* F5 */   set_6_iY    ,  /* ?? */
/* F6 */   set_6_piy   ,
/* F7 */   set_6_piya  ,
/* F8 */   set_7_piyb  ,
/* F9 */   set_7_piyc  ,
/* FA */   set_7_piyd  ,
/* FB */   set_7_piye  ,
/* FC */   set_7_Iy    ,  /* ?? */
/* FD */   set_7_iY    ,  /* ?? */
/* FE */   set_7_piy   ,
/* FF */   set_7_piya
};

static void (*instruc_tablED[256])() =
{
/* 00 */   dummyED     ,
/* 01 */   dummyED     ,
/* 02 */   dummyED     ,
/* 03 */   dummyED     ,
/* 04 */   dummyED     ,
/* 05 */   dummyED     ,
/* 06 */   dummyED     ,
/* 07 */   dummyED     ,
/* 08 */   dummyED     ,
/* 09 */   dummyED     ,
/* 0A */   dummyED     ,
/* 0B */   dummyED     ,
/* 0C */   dummyED     ,
/* 0D */   dummyED     ,
/* 0E */   dummyED     ,
/* 0F */   dummyED     ,
/* 10 */   dummyED     ,
/* 11 */   dummyED     ,
/* 12 */   dummyED     ,
/* 13 */   dummyED     ,
/* 14 */   dummyED     ,
/* 15 */   dummyED     ,
/* 16 */   dummyED     ,
/* 17 */   dummyED     ,
/* 18 */   dummyED     ,
/* 19 */   dummyED     ,
/* 1A */   dummyED     ,
/* 1B */   dummyED     ,
/* 1C */   dummyED     ,
/* 1D */   dummyED     ,
/* 1E */   dummyED     ,
/* 1F */   dummyED     ,
/* 20 */   dummyED     ,
/* 21 */   dummyED     ,
/* 22 */   dummyED     ,
/* 23 */   dummyED     ,
/* 24 */   dummyED     ,
/* 25 */   dummyED     ,
/* 26 */   dummyED     ,
/* 27 */   dummyED     ,
/* 28 */   dummyED     ,
/* 29 */   dummyED     ,
/* 2A */   dummyED     ,
/* 2B */   dummyED     ,
/* 2C */   dummyED     ,
/* 2D */   dummyED     ,
/* 2E */   dummyED     ,
/* 2F */   dummyED     ,
/* 30 */   dummyED     ,
/* 31 */   dummyED     ,
/* 32 */   dummyED     ,
/* 33 */   dummyED     ,
/* 34 */   dummyED     ,
/* 35 */   dummyED     ,
/* 36 */   dummyED     ,
/* 37 */   dummyED     ,
/* 38 */   dummyED     ,
/* 39 */   dummyED     ,
/* 3A */   dummyED     ,
/* 3B */   dummyED     ,
/* 3C */   dummyED     ,
/* 3D */   dummyED     ,
/* 3E */   dummyED     ,
/* 3F */   dummyED     ,
/* 40 */   in_b_pc     ,
/* 41 */   out_pc_b    ,
/* 42 */   sbc_hl_bc   ,
/* 43 */   ld_pnn_bc   ,
/* 44 */   neg         ,
/* 45 */   retn        ,
/* 46 */   im_0        ,
/* 47 */   ld_i_a      ,
/* 48 */   in_c_pc     ,
/* 49 */   out_pc_c    ,
/* 4A */   adc_hl_bc   ,
/* 4B */   ld_bc_pnn   ,
/* 4C */   neg         ,   /* Undocumented by Zilog */
/* 4D */   reti        ,
/* 4E */   im_0        ,   /* Undocumented by Zilog */
/* 4F */   ld_r_a      ,
/* 50 */   in_d_pc     ,
/* 51 */   out_pc_d    ,
/* 52 */   sbc_hl_de   ,
/* 53 */   ld_pnn_de   ,
/* 54 */   neg         ,   /* Undocumented by Zilog */
/* 55 */   retn /*ret*/,   /* Undocumented by Zilog */
/* 56 */   im_1        ,
/* 57 */   ld_a_i      ,
/* 58 */   in_e_pc     ,
/* 59 */   out_pc_e    ,
/* 5A */   adc_hl_de   ,
/* 5B */   ld_de_pnn   ,
/* 5C */   neg         ,   /* Undocumented by Zilog */
/* 5D */   retn /*ret*/,   /* Undocumented by Zilog */
/* 5E */   im_2        ,
/* 5F */   ld_a_r      ,
/* 60 */   in_h_pc     ,
/* 61 */   out_pc_h    ,
/* 62 */   sbc_hl_hl   ,
/* 63 */   ld_pnn_hlED ,   /* Undocumented by Zilog */
/* 64 */   neg         ,   /* Undocumented by Zilog */
/* 65 */   retn /*ret*/,   /* Undocumented by Zilog */
/* 66 */   im_0        ,   /* Undocumented by Zilog */
/* 67 */   rrd         ,
/* 68 */   in_l_pc     ,
/* 69 */   out_pc_l    ,
/* 6A */   adc_hl_hl   ,
/* 6B */   ld_hl_pnnED ,   /* Undocumented by Zilog */
/* 6C */   neg         ,   /* Undocumented by Zilog */
/* 6D */   retn /*ret*/,   /* Undocumented by Zilog */
/* 6E */   im_0        ,   /* Undocumented by Zilog */
/* 6F */   rld         ,
/* 70 */   in_f_pc     ,   /* Undocumented by Zilog */
/* 71 */   out_pc_f    ,   /* Undocumented by Zilog */
/* 72 */   sbc_hl_sp   ,
/* 73 */   ld_pnn_sp   ,
/* 74 */   neg         ,   /* Undocumented by Zilog */
/* 75 */   retn /*ret*/,   /* Undocumented by Zilog */
/* 76 */   im_1        ,   /* Undocumented by Zilog */
/* 77 */   dummyED     ,
/* 78 */   in_a_pc     ,
/* 79 */   out_pc_a    ,
/* 7A */   adc_hl_sp   ,
/* 7B */   ld_sp_pnn   ,
/* 7C */   neg         ,   /* Undocumented by Zilog */
/* 7D */   retn /*ret*/,   /* Undocumented by Zilog */
/* 7E */   im_2        ,   /* Undocumented by Zilog */
/* 7F */   dummyED     ,
/* 80 */   dummyED     ,
/* 81 */   dummyED     ,
/* 82 */   dummyED     ,
/* 83 */   dummyED     ,
/* 84 */   dummyED     ,
/* 85 */   dummyED     ,
/* 86 */   dummyED     ,
/* 87 */   dummyED     ,
/* 88 */   dummyED     ,
/* 89 */   dummyED     ,
/* 8A */   dummyED     ,
/* 8B */   dummyED     ,
/* 8C */   dummyED     ,
/* 8D */   dummyED     ,
/* 8E */   dummyED     ,
/* 8F */   dummyED     ,
/* 90 */   dummyED     ,
/* 91 */   dummyED     ,
/* 92 */   dummyED     ,
/* 93 */   dummyED     ,
/* 94 */   dummyED     ,
/* 95 */   dummyED     ,
/* 96 */   dummyED     ,
/* 97 */   dummyED     ,
/* 98 */   dummyED     ,
/* 99 */   dummyED     ,
/* 9A */   dummyED     ,
/* 9B */   dummyED     ,
/* 9C */   dummyED     ,
/* 9D */   dummyED     ,
/* 9E */   dummyED     ,
/* 9F */   dummyED     ,
/* A0 */   ldi         ,
/* A1 */   cpi         ,
/* A2 */   ini         ,
/* A3 */   outi        ,
/* A4 */   dummyED     ,
/* A5 */   dummyED     ,
/* A6 */   dummyED     ,
/* A7 */   dummyED     ,
/* A8 */   ldd         ,
/* A9 */   cpd         ,
/* AA */   ind         ,
/* AB */   outd        ,
/* AC */   dummyED     ,
/* AD */   dummyED     ,
/* AE */   dummyED     ,
/* AF */   dummyED     ,
/* B0 */   ldir        ,
/* B1 */   cpir        ,
/* B2 */   inir        ,
/* B3 */   otir        ,
/* B4 */   dummyED     ,
/* B5 */   dummyED     ,
/* B6 */   dummyED     ,
/* B7 */   dummyED     ,
/* B8 */   lddr        ,
/* B9 */   cpdr        ,
/* BA */   indr        ,
/* BB */   otdr        ,
/* BC */   dummyED     ,
/* BD */   dummyED     ,
/* BE */   dummyED     ,
/* BF */   dummyED     ,
/* C0 */   dummyED     ,
/* C1 */   dummyED     ,
/* C2 */   dummyED     ,
/* C3 */   dummyED     ,
/* C4 */   dummyED     ,
/* C5 */   dummyED     ,
/* C6 */   dummyED     ,
/* C7 */   dummyED     ,
/* C8 */   dummyED     ,
/* C9 */   dummyED     ,
/* CA */   dummyED     ,
/* CB */   dummyED     ,
/* CC */   dummyED     ,
/* CD */   dummyED     ,
/* CE */   dummyED     ,
/* CF */   dummyED     ,
/* D0 */   dummyED     ,
/* D1 */   dummyED     ,
/* D2 */   dummyED     ,
/* D3 */   dummyED     ,
/* D4 */   dummyED     ,
/* D5 */   dummyED     ,
/* D6 */   dummyED     ,
/* D7 */   dummyED     ,
/* D8 */   dummyED     ,
/* D9 */   dummyED     ,
/* DA */   dummyED     ,
/* DB */   dummyED     ,
/* DC */   dummyED     ,
/* DD */   dummyED     ,
/* DE */   dummyED     ,
/* DF */   dummyED     ,
/* E0 */   dummyED     ,
/* E1 */   dummyED     ,
/* E2 */   dummyED     ,
/* E3 */   dummyED     ,
/* E4 */   dummyED     ,
/* E5 */   dummyED     ,
/* E6 */   dummyED     ,
/* E7 */   dummyED     ,
/* E8 */   dummyED     ,
/* E9 */   dummyED     ,
/* EA */   dummyED     ,
/* EB */   dummyED     ,
/* EC */   dummyED     ,
/* ED */   dummyED     ,
/* EE */   dummyED     ,
/* EF */   dummyED     ,
/* F0 */   dummyED     ,
/* F1 */   dummyED     ,
/* F2 */   dummyED     ,
/* F3 */   dummyED     ,
/* F4 */   dummyED     ,
/* F5 */   dummyED     ,
/* F6 */   dummyED     ,
/* F7 */   dummyED     ,
/* F8 */   dummyED     ,
/* F9 */   dummyED     ,
/* FA */   dummyED     ,
/* FB */   level_loader_trap,
/* FC */   dummyED     ,
/* FD */   dummyED     ,
/* FE */   dummyED     ,
/* FF */   dummyED
};

/* EOF: IVars.H */
